@@ -2,15 +2,8 @@
 
 from __future__ import annotations
 
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Callable,
-    List,
-    Union,
-    cast,
-    overload,
-)
+from contextlib import suppress
+from typing import TYPE_CHECKING, Any, Callable, Dict, List, Union, cast, overload
 
 import peewee as pw
 from playhouse.migrate import (
@@ -28,110 +21,20 @@ from playhouse.migrate import PostgresqlMigrator as PgM
 from playhouse.migrate import SchemaMigrator as ScM
 from playhouse.migrate import SqliteMigrator as SqM
 
+from peewee_migrate.utils import depricated_method
+
 from .logs import logger
 
 if TYPE_CHECKING:
     from .types import TModelType, TVModelType
 
 
-class SchemaMigrator(ScM):
-    """Implement migrations."""
-
-    @classmethod
-    def from_database(cls, database: Union[pw.Database, pw.Proxy]) -> SchemaMigrator:
-        """Initialize migrator by db."""
-        if isinstance(database, PostgresqlDatabase):
-            return PostgresqlMigrator(database)
-
-        if isinstance(database, SqliteDatabase):
-            return SqliteMigrator(database)
-
-        if isinstance(database, MySQLDatabase):
-            return MySQLMigrator(database)
-
-        raise ValueError("Unsupported database: %s" % database)
-
-    def drop_table(self, model: TModelType, *, cascade: bool = True) -> Callable[[], Any]:
-        """Drop table."""
-        return lambda: model.drop_table(cascade=cascade)
-
-    @operation
-    def change_column(
-        self, table: str, column_name: str, field: pw.Field
-    ) -> List[Union[Context, Operation]]:
-        """Change column."""
-        operations: List[Union[Context, Operation]] = [
-            self.alter_change_column(table, column_name, field)
-        ]
-        if not field.null:
-            operations.extend([self.add_not_null(table, column_name)])
-        return operations
-
-    def alter_change_column(self, table: str, column: str, field: pw.Field) -> Context:
-        """Support change columns."""
-        ctx = self.make_context()
-        field_null, field.null = field.null, True
-        ctx = self._alter_table(ctx, table).literal(" ALTER COLUMN ").sql(field.ddl(ctx))
-        field.null = field_null
-        return ctx
-
-    @operation
-    def sql(self, sql: str, *params) -> SQL:
-        """Execute raw SQL."""
-        return SQL(sql, *params)
-
-    def alter_add_column(
-        self, table: str, column_name: str, field: pw.Field, **kwargs
-    ) -> Operation:
-        """Fix fieldname for ForeignKeys."""
-        name = field.name
-        op = super(SchemaMigrator, self).alter_add_column(table, column_name, field, **kwargs)
-        if isinstance(field, pw.ForeignKeyField):
-            field.name = name
-        return op
-
-
-class MySQLMigrator(SchemaMigrator, MqM):
-    """Support MySQL."""
-
-    def alter_change_column(self, table: str, column: str, field: pw.Field) -> Context:
-        """Support change columns."""
-        ctx = self.make_context()
-        field_null, field.null = field.null, True
-        ctx = self._alter_table(ctx, table).literal(" MODIFY COLUMN ").sql(field.ddl(ctx))
-        field.null = field_null
-        return ctx
-
-
-class PostgresqlMigrator(SchemaMigrator, PgM):
-    """Support the migrations in postgresql."""
-
-    def alter_change_column(self, table: str, column_name: str, field: pw.Field) -> Context:
-        """Support change columns."""
-        context = super(PostgresqlMigrator, self).alter_change_column(table, column_name, field)
-        context._sql.insert(-1, "TYPE")  # type: ignore[]
-        context._sql.insert(-1, " ")  # type: ignore[]
-        return context
-
-
-class SqliteMigrator(SchemaMigrator, SqM):
-    """Support the migrations in sqlite."""
-
-    def drop_table(self, model: pw.Model, *, cascade: bool = True) -> Callable:
-        """Sqlite doesnt support cascade syntax by default."""
-        return lambda: model.drop_table(cascade=False)
-
-    def alter_change_column(self, table: str, column: str, field: pw.Field) -> Operation:
-        """Support change columns."""
-        return self._update_column(table, column, lambda a, b: b)  # type: ignore[]
-
-
 class ORM:
     __slots__ = ("__tables__", "__models__")
 
-    def __init__(self):
-        self.__tables__ = {}
-        self.__models__ = {}
+    def __init__(self: ORM):
+        self.__tables__: Dict[str, TModelType] = {}
+        self.__models__: Dict[str, TModelType] = {}
 
     def add(self, model: TModelType):
         self.__models__[model.__name__] = model
@@ -180,6 +83,14 @@ class Migrator:
         """Iterate over models."""
         return iter(self.orm)
 
+    def fake(self):
+        """Sync the current snapshot but not run operations.
+
+        >> with migrator.sync():
+        >>  # ... do changes
+        """
+        return SyncContext(self)
+
     @overload
     def __get_model__(self, model: TVModelType) -> TVModelType:
         ...
@@ -205,14 +116,18 @@ class Migrator:
         op = cast(Operation, self.__migrator__.sql(sql, *params))
         self.__ops__.append(op)
 
-    def python(self, func: Callable, *args, **kwargs):
+    def run(self, func: Callable, *args, **kwargs):
         """Run a python function."""
         self.__ops__.append(lambda: func(*args, **kwargs))
 
-    def create_table(self, model: TVModelType) -> TVModelType:
+    python = depricated_method(run, "python")
+
+    def create_model(self, model: TVModelType) -> TVModelType:
         """Create model and table in database.
 
-        >> migrator.create_table(model)
+        :param model: Model class
+
+        >> migrator.create_model(Model)
         """
         meta = model._meta  # type: ignore[]
         self.orm.add(model)
@@ -221,42 +136,53 @@ class Migrator:
         self.__ops__.append(model.create_table)
         return model
 
-    create_model = create_table
+    create_table = depricated_method(create_model, "create_table")
 
-    def drop_table(self, model: Union[str, TModelType], *, cascade: bool = True):
+    def remove_model(self, model: Union[str, TModelType], *, cascade: bool = True):
         """Drop model and table from database.
 
-        >> migrator.drop_table(model, cascade=True)
+        :param model: Model class or table name
+        :param cascade(true): Drop table with cascade
+
+        >> migrator.remove_model(Model, cascade=True)
         """
         model = self.__get_model__(model)
         self.orm.remove(model)
         self.__ops__.append(self.__migrator__.drop_table(model, cascade=cascade))
 
-    remove_model = drop_table
+    drop_table = depricated_method(remove_model, "drop_table")
 
-    def add_columns(self, model: Union[str, TModelType], **fields: pw.Field) -> TModelType:
-        """Create new fields."""
+    def add_fields(self, model: Union[str, TModelType], **fields: pw.Field) -> TModelType:
+        """Change fields.
+
+        :param model: Model class or table name
+
+        >> migrator.change_fields(Model, name=pw.CharField(null=True))
+        """
         model = self.__get_model__(model)
         meta = model._meta  # type: ignore[]
         for name, field in fields.items():
             meta.add_field(name, field)
+
             self.__ops__.append(
                 self.__migrator__.add_column(  # type: ignore[]
                     meta.table_name, field.column_name, field
                 )
             )
-            if field.unique:
-                self.__ops__.append(
-                    self.__migrator__.add_index(meta.table_name, (field.column_name,), unique=True)
-                )
+
         return model
 
-    add_fields = add_columns
+    add_columns = depricated_method(add_fields, "add_columns")
 
-    def change_columns(self, model: Union[str, TModelType], **fields: pw.Field) -> TModelType:
-        """Change fields."""
+    def change_fields(self, model: Union[str, TModelType], **fields: pw.Field) -> TModelType:
+        """Change fields.
+
+        :param model: Model class or table name
+
+        >> migrator.change_fields(Model, name=pw.CharField(null=True))
+        """
         model = self.__get_model__(model)
-        meta = model._meta  # type: ignore[]
+        meta: pw.Metadata = model._meta  # type: ignore[]
         for name, field in fields.items():
             old_field = meta.fields.get(name, field)
             old_column_name = old_field and old_field.column_name
@@ -301,21 +227,28 @@ class Migrator:
 
             if field.unique:
                 index = (field.column_name,), field.unique
-                self.__ops__.append(self.__migrator__.add_index(meta.table_name, *index))
                 meta.indexes.append(index)
+                self.__ops__.append(self.__migrator__.add_index(meta.table_name, *index))
             else:
                 index = field.column_name
+                with suppress(ValueError):
+                    meta.indexes.remove(((field.column_name,), True))
                 self.__ops__.append(self.__migrator__.drop_index(meta.table_name, index))
-                meta.indexes.remove(((field.column_name,), old_field.unique))
 
         return model
 
-    change_fields = change_columns
+    change_columns = depricated_method(change_fields, "change_columns")
 
-    def drop_columns(
+    def remove_fields(
         self, model: Union[str, TModelType], *names: str, cascade: bool = True
     ) -> TModelType:
-        """Remove fields from model."""
+        """Remove fields from model.
+
+        :param model: Model class or table name
+        :param cascade(true): Drop columns with cascade
+
+        >> migrator.remove_fields(Model, "name", "age", cascade=True)
+        """
         model = self.__get_model__(model)
         meta = model._meta  # type: ignore[]
         fields = [field for field in meta.fields.values() if field.name in names]
@@ -331,25 +264,19 @@ class Migrator:
             )
         return model
 
-    remove_fields = drop_columns
+    drop_columns = depricated_method(remove_fields, "drop_columns")
 
-    def __del_field__(self, model: TModelType, field: pw.Field):
-        """Delete field from model."""
-        meta = model._meta  # type: ignore[]
-        meta.remove_field(field.name)
-        delattr(model, field.name)
-        if isinstance(field, pw.ForeignKeyField):
-            obj_id_name = field.column_name
-            if field.column_name == field.name:
-                obj_id_name += "_id"
-            if hasattr(model, obj_id_name):
-                delattr(model, obj_id_name)
-            delattr(field.rel_model, field.backref)
-
-    def rename_column(
+    def rename_field(
         self, model: Union[str, TModelType], old_name: str, new_name: str
     ) -> TModelType:
-        """Rename field in model."""
+        """Rename field in model.
+
+        :param model: Model class or table name
+        :param old_name: Old field name
+        :param new_name: New field name
+
+        >> migrator.rename_field(Model, "name", "full_name")
+        """
         model = self.__get_model__(model)
         meta = model._meta  # type: ignore[]
         field = meta.fields[old_name]
@@ -365,7 +292,20 @@ class Migrator:
         )
         return model
 
-    rename_field = rename_column
+    rename_column = depricated_method(rename_field, "rename_column")
+
+    def __del_field__(self, model: TModelType, field: pw.Field):
+        """Delete field from model."""
+        meta = model._meta  # type: ignore[]
+        meta.remove_field(field.name)
+        delattr(model, field.name)
+        if isinstance(field, pw.ForeignKeyField):
+            obj_id_name = field.column_name
+            if field.column_name == field.name:
+                obj_id_name += "_id"
+            if hasattr(model, obj_id_name):
+                delattr(model, obj_id_name)
+            delattr(field.rel_model, field.backref)
 
     def rename_table(self, model: Union[str, TModelType], new_name: str) -> TModelType:
         """Rename table in database."""
@@ -444,3 +384,154 @@ class Migrator:
         meta.defaults[field] = field.default = default
         self.__ops__.append(self.__migrator__.apply_default(meta.table_name, name, field))
         return model
+
+    def add_constraint(self, model: Union[str, TModelType], name, constraint):
+        """Add constraint."""
+        model = self.__get_model__(model)
+        meta = model._meta  # type: ignore[]
+        self.__ops__.append(self.__migrator__.add_constraint(meta.table_name, name, constraint))
+        return model
+
+    def drop_constraints(self, model: Union[str, TModelType], *names: str) -> TModelType:
+        """Drop constraints."""
+        model = self.__get_model__(model)
+        meta = model._meta  # type: ignore[]
+        self.__ops__.extend(
+            [self.__migrator__.drop_constraint(meta.table_name, name) for name in names]
+        )
+        return model
+
+
+class SchemaMigrator(ScM):
+    """Implement migrations."""
+
+    @classmethod
+    def from_database(cls, database: Union[pw.Database, pw.Proxy]) -> SchemaMigrator:
+        """Initialize migrator by db."""
+        if isinstance(database, PostgresqlDatabase):
+            return PostgresqlMigrator(database)
+
+        if isinstance(database, SqliteDatabase):
+            return SqliteMigrator(database)
+
+        if isinstance(database, MySQLDatabase):
+            return MySQLMigrator(database)
+
+        raise ValueError("Unsupported database: %s" % database)
+
+    def drop_table(self, model: TModelType, *, cascade: bool = True) -> Callable[[], Any]:
+        """Drop table."""
+        return lambda: model.drop_table(cascade=cascade)
+
+    @operation
+    def sql(self, sql: str, *params) -> SQL:
+        """Execute raw SQL."""
+        return SQL(sql, *params)
+
+    @operation
+    def change_column(
+        self, table: str, column_name: str, field: pw.Field
+    ) -> List[Union[Context, Operation]]:
+        """Change column."""
+        operations: List[Union[Context, Operation]] = self.alter_change_column(
+            table, column_name, field
+        )
+        if not field.null:
+            operations.append(self.add_not_null(table, column_name))
+        return operations
+
+    @operation
+    def add_default(self, table: str, column: str, field: pw.Field):
+        default = field.default
+        if callable(default):
+            default = default()
+        alter_column: pw.Context = self._alter_column(self.make_context(), table, column)
+        ctx: pw.Context = alter_column.literal(" SET DEFAULT ")
+        return ctx.sql(field.db_value(default))
+
+    def alter_change_column(
+        self, table: str, column: str, field: pw.Field
+    ) -> List[Union[Context, Operation]]:
+        """Support change columns."""
+        ctx = self.make_context()
+        field_null, field.null = field.null, True
+        ctx = self._alter_table(ctx, table).literal(" ALTER COLUMN ").sql(field.ddl(ctx))
+        field.null = field_null
+        return [ctx]
+
+    def alter_add_column(
+        self, table: str, column_name: str, field: pw.Field, **kwargs
+    ) -> Operation:
+        """Fix fieldname for ForeignKeys."""
+        name = field.name
+        op = super(SchemaMigrator, self).alter_add_column(table, column_name, field, **kwargs)
+        if isinstance(field, pw.ForeignKeyField):
+            field.name = name
+        return op
+
+
+class MySQLMigrator(SchemaMigrator, MqM):
+    """Support MySQL."""
+
+    def alter_change_column(
+        self, table: str, column: str, field: pw.Field
+    ) -> List[Union[Context, Operation]]:
+        """Support change columns."""
+        ctx = self.make_context()
+        field_null, field.null = field.null, True
+        ctx = self._alter_table(ctx, table).literal(" MODIFY COLUMN ").sql(field.ddl(ctx))
+        field.null = field_null
+        return [ctx]
+
+
+class PostgresqlMigrator(SchemaMigrator, PgM):
+    """Support the migrations in postgresql."""
+
+    def alter_change_column(
+        self, table: str, column: str, field: pw.Field
+    ) -> List[Union[Context, Operation]]:
+        """Support change columns."""
+        ctx = self.make_context()
+        fn, field.null = field.null, True
+        fc, field.constraints = field.constraints, []
+        ddl = field.ddl(ctx)
+        ddl.nodes.insert(1, pw.SQL("TYPE"))
+        ctx = self._alter_table(ctx, table).literal(" ALTER COLUMN ").sql(ddl)
+        field.null, field.constraints = fn, fc
+        res = [ctx]
+        if field.default is not None:
+            res.append(self.add_default(table, column, field))
+        return res
+
+
+class SqliteMigrator(SchemaMigrator, SqM):
+    """Support the migrations in sqlite."""
+
+    def drop_table(self, model: pw.Model, *, cascade: bool = True) -> Callable:
+        """Sqlite doesnt support cascade syntax by default."""
+        return lambda: model.drop_table(cascade=False)
+
+    def alter_change_column(
+        self, table: str, column: str, field: pw.Field
+    ) -> List[Union[Operation, Context]]:
+        """Support change columns."""
+
+        def fn(c_name, c_def):
+            ctx = self.make_context()
+            ctx.sql(field.ddl(ctx))
+            return ctx.query()[0]
+
+        return [self._update_column(table, column, fn)]  # type: ignore[]
+
+
+class SyncContext:
+    def __init__(self, migrator):
+        self.migrator = migrator
+        self.ops = None
+
+    def __enter__(self):
+        self.ops = list(self.migrator.__ops__)
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.migrator.__ops__ = self.ops
