@@ -1,11 +1,12 @@
+"""Peewee-Migrate custom types."""
+
 from __future__ import annotations
 
-from typing import Any, Dict, Type, TypeVar, Union
+from typing import Any, Dict, Type, TypeVar
 
 from peewee import Model
 
 TModelType = Type[Model]
-TModelArg = Union[TModelType, str]
 TParams = Dict[str, Any]
 
 TVModelType = TypeVar("TVModelType", bound=TModelType)
